/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BaseReader.h"

#include <iostream>
#include <stdexcept>

using namespace std;

namespace lstdpp128 {

BaseReader::BaseReader() :
	_size(0),
	_sizeInBytes(0),
	_buffer(0),
	_version(0),
	_timeBase(0),
	_totalNumberOfBoards(0),
	_boardHeader(0) {
}

BaseReader::~BaseReader() {

	delete[] _buffer;
}

void BaseReader::init(int32_t size) {
	_size = size;
	_sizeInBytes = size * sizeof(int32_t);
	_buffer = new int32_t[size];

	// check that size is a multiple of a binary event
	if (size % EVENT_SIZE_32 != 0) {
		ostringstream os;
		os << "buffer size must be a multiple of " << EVENT_SIZE_32;
		throw runtime_error(os.str());
	}
}

int32_t * BaseReader::buffer() const {
	return _buffer;
}


uint32_t BaseReader::version() const {
	return _version;
}

uint32_t BaseReader::timeBase() const {
	return _timeBase;
}

uint32_t BaseReader::totalNumberOfBoards() const {
	return _totalNumberOfBoards;
}

}
