/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_CHECK_H
#define LSTDPP_CHECK_H

#include <cstring>
#include <algorithm>
#include <fstream>
#include <vector>
#include <time.h>

#include "Event.h"
#include "EventBlockArray.h"

namespace lstdpp128 {

class Check {

public:
	Check();
	~Check();

	void init();

	int32_t numberOfGoodEvents() const;
	int32_t numberOfBadEvents() const;
	int32_t numberOfEventsNotInserted() const;
	time_type maxTime() const;
	bool isOrdered() const;

	/**
	 * Functions to implement.
	 */
	bool checkEvent(Event const & event);
	void processBadEvent(Event const & event);
	void processEvent(Event const & event);
	void processEvent(EventBlockArrayIterator const & e);
	void processEventNotInserted(Event const & event);

	template<typename Iterator>
	Iterator processBlock(Iterator first, Iterator last, bool lastBlock);
	void finish();

private:
	int32_t _maxNumberOfChannels;

	int32_t _numberOfGoodEvents;
	int32_t _numberOfBadEvents;
	int32_t _numberOfEventsNotInserted;

	bool _onZeroBlock;
	int32_t _zeroCounter;

	Event _lastEvent;
	time_type _maxTime;

	bool _ordered;

	static const int16_t MAX_BITS = 15;
	static const int16_t GLOBAL = 0;
	static const int16_t GLOBAL_CLEAN = 1;
	static const int16_t DETECTOR = 2;
	static const int16_t DETECTOR_CLEAN = 3;
};

template<typename Iterator>
Iterator Check::processBlock(Iterator first, Iterator last, bool lastBlock) {
	return last;
}

}

#endif
