/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "EventArray.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace lstdpp128 {

bool EventArray::isInsertable(time_type time) const {

//	if (time < _currentMaxProcessTime) {
//		cout << "not inserted " << time << " ? " << _currentMaxProcessTime << endl;
//	}

	return time >= _currentMaxProcessTime;
}

EventArray::InsertResult EventArray::insertEvent(Event const & event) {

	*_e = event;
	++_e;

	if (_init && _e == _end || !_init && _e == _processedCenter) {

		// updating max process time
		_currentMaxProcessTime = _processedCenter->time();

		// sorting events
		sort(_first, _end, CompareEvents());

		//cout << "sort " << _array.front().time << " - " << _array.back().time << " max " << _currentMaxProcessTime << endl;

		_init = false;
		_e = _first;

		return InsertResult(_first, _center);
	}

	return InsertResult();
}

void EventArray::pack() {

	if (!_init) {
		// copying elements of the second part
		for (array_type::const_iterator c = _processedCenter; c != _end; ++c) {
			*_e = *c;
			++_e;
		}
	}

	// packing the array
	_array.resize(distance(_first, _e));

	// sorting events
	sort(_first, _array.end(), CompareEvents());

//	cout << "array packed" << endl;
//
//	for (array_type::const_iterator e = _array.begin(); e != _array.end(); ++e) {
//		cout << *e << endl;
//	}
//
//	cout << "end array packed" << endl;
}

}
