/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_EVENTARRAY_H
#define LSTDPP_EVENTARRAY_H

#include <vector>

#include "Event.h"

namespace lstdpp128 {

/**
 * The EventArray is made up of two blocks of events. The blocks are contiguous to ensure fast access.
 */
class EventArray {

public:
	typedef std::vector<Event> array_type;

	/**
	 * The blockSize is the number of events of a block.
	 */
	explicit EventArray(int32_t blockSize) :
		_blockSize(blockSize),
		_array(2 * blockSize, Event()),
		_init(true),
		_currentMaxProcessTime(0) {

		_first = _array.begin();
		_center = _first + blockSize;
		_end = _array.end();

		_processedCenter = _center;

		_e = _first;

	}

	~EventArray() {
	}

	bool isInsertable(time_type time) const;

	inline array_type & array() {
		return _array;
	}

	inline array_type const & array() const {
		return _array;
	}

	inline array_type::iterator center() const {
		return _center;
	}

	inline void setProcessedCenter(array_type::iterator center) {
		_processedCenter = center;
	}

	struct InsertResult {
		bool process;
		array_type::iterator begin;
		array_type::iterator end;

		InsertResult() :
			process(false) {
		}

		InsertResult(array_type::iterator a, array_type::iterator b) :
			process(true), begin(a), end(b) {
		}
	};

	InsertResult insertEvent(Event const & event);
	void pack();

private:
	int32_t _blockSize;
	array_type _array;

	array_type::iterator _first;
	array_type::iterator _center;
	array_type::iterator _end;

	array_type::iterator _processedCenter;

	bool _init;

	time_type _currentMaxProcessTime;
	array_type::iterator _e;
};

}

#endif
