/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_EVENTBLOCKARRAYALGORITHM_H
#define LSTDPP_EVENTBLOCKARRAYALGORITHM_H

#include <iostream>
#include "BaseAlgorithm.h"
#include "EventBlockArray.h"

namespace lstdpp128 {

template<typename Processor>
class EventBlockArrayAlgorithm : public BaseAlgorithm {

public:
	explicit EventBlockArrayAlgorithm();
	virtual ~EventBlockArrayAlgorithm();

	Processor & processor();

	void process(int32_t * buffer, int32_t size, EventBlockArray & blocks);
	void finish(EventBlockArray & blocks);

private:
	Processor _processor;
};

template<typename Processor>
EventBlockArrayAlgorithm<Processor>::EventBlockArrayAlgorithm() :
	BaseAlgorithm() {
}

template<typename Processor>
EventBlockArrayAlgorithm<Processor>::~EventBlockArrayAlgorithm() {
}

template<typename Processor>
Processor & EventBlockArrayAlgorithm<Processor>::processor() {
	return _processor;
}

template<typename Processor>
void EventBlockArrayAlgorithm<Processor>::process(int32_t * buffer, int32_t size, EventBlockArray & blocks) {

	// reading next block
	int32_t index = 0;
	int32_t * bufferPtr = buffer;

	while (index < size) {
		// reading event
		Event event;

		if (!readEvent(event, bufferPtr + index)) {
			_processor.processBadEvent(event);
			continue;
		}

		if (!_processor.checkEvent(event)) {
			_processor.processBadEvent(event);

		} else if (blocks.isInsertable(event.time())) {
			EventBlockArrayIterator e = blocks.insertEvent(event);
			_processor.processEvent(e);

			// reducing blocks if necessary
			EventBlock * block = blocks.updateBlocks();

			if (block != 0) {
				_processor.processBlock(block->events.begin(), block->events.end(), true);
			}

			delete block;

		} else {
			_processor.processEventNotInserted(event);

		}

		// jump 128 bits
		index += EVENT_SIZE_32;
	}
}

template<typename Processor>
void EventBlockArrayAlgorithm<Processor>::finish(EventBlockArray & blocks) {

	for (EventBlockArray::array_type::const_iterator b = blocks.array().begin(); b != blocks.array().end(); ++b) {
		_processor.processBlock((*b)->events.begin(), (*b)->events.end(), true);
	}

	_processor.finish();
}

}

#endif
