/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_EVENTDATA_H
#define LSTDPP_EVENTDATA_H

#include "Event.h"

namespace lstdpp128 {

	// Data.
	uint32_t eventData(Event const & event);
	uint32_t eventDataPlus(Event const & event);


	// Energy functions.
	bool boardHasEnergy(BoardType type);
	bool eventHasEnergy(Event const & event);
	uint16_t boardEnergyResolution(BoardType type);
	uint16_t eventEnergyResolution(Event const & event);
	uint16_t eventEnergy(Event const & event);


	// qShort, qLong
	bool boardHasQ(BoardType type);
	bool eventHasQ(Event const & event);
	uint16_t boardQShortResolution(BoardType type);
	uint16_t eventQShortResolution(Event const & event);
	uint16_t boardQLongResolution(BoardType type);
	uint16_t eventQLongResolution(Event const & event);
	uint16_t eventQShort(Event const & event);
	uint16_t eventQLong(Event const & event);
	float eventQRatio(Event const & event);


	// Position X and Y functions.
	bool boardHasPositionX(BoardType type);
	bool eventHasPositionX(Event const & event);
	bool boardHasPositionY(BoardType type);
	bool eventHasPositionY(Event const & event);
	uint16_t boardPositionXResolution(BoardType type);
	uint16_t eventPositionXResolution(Event const & event);
	uint16_t boardPositionYResolution(BoardType type);
	uint16_t eventPositionYResolution(Event const & event);
	uint16_t eventPositionX(Event const & event);
	uint16_t eventPositionY(Event const & event);


	// A, B functions.
	bool boardHasAB(BoardType type);
	bool eventHasAB(Event const & event);
	uint16_t boardAResolution(BoardType type);
	uint16_t boardAplusBResolution(BoardType type);
	uint16_t eventAResolution(Event const & event);
	uint16_t eventAplusBResolution(Event const & event);
	uint16_t eventA(Event const & event);
	uint16_t eventAplusB(Event const & event);


	// Zero-crossing.
	bool boardHasZeroCrossing(BoardType type);
	bool eventHasZeroCrossing(Event const & event);
	uint16_t boardZeroCrossingResolution(BoardType type);
	uint16_t eventZeroCrossingResolution(Event const & event);
	float eventZeroCrossing(Event const & event);


	// Special data for boards which provide unspecified data (content is not specified).
	bool boardHasUnspecifiedData(BoardType type);
	bool eventHasUnspecifiedData(Event const & event);
	uint16_t boardUnspecifiedDataResolution(BoardType type);
	uint32_t eventUnspecifiedDataResolution(Event const & event);
	uint32_t eventUnspecifiedData(Event const & event);

	// Flags.
	bool boardHasFlags(BoardType type);
	bool eventHasFlags(Event const & event);

	/**
	 * Flags for the version 2:
	 * 15 : WE : Waveform Enabled
	 * 14 : PU : Pile Up
	 * 13 : DT : Dead Time (1724 PHA) / Lost Event (1725/30 PHA) / Trigger Lost (1725/30 PSD)
	 * 12 : OR : Over-Range / Saturation
	 * 11 : BF : Board Fail (formerly Trigger Lost)
	 * 10 : NT : N Trigger Counted
	 * 09 : NL : N Lost Trigger Counted
	 * 08 : MC : MATCH_COINC (1725 PHA and 1730 PHA)
 	 * 07 : NC : NOT_MATCH_COINC (1725 PHA and 1730 PHA)
	 * 06..0   : Reserved
	 */
	uint16_t eventFlags(Event const & event);

	// Output for Event.
	std::string eventDataColumnsToString(BoardType type);
	std::string eventDataToString(Event const & event);
}

#endif
