/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_INSERT_H
#define LSTDPP_INSERT_H

#include <cstring>
#include <algorithm>
#include <fstream>
#include <map>
#include <time.h>

#include "Event.h"
#include "EventBlockArray.h"

namespace lstdpp128 {

class Insert {

public:
	Insert();
	~Insert();

	void init();

	int32_t numberOfGoodEvents() const;
	int32_t numberOfBadEvents() const;
	int32_t numberOfEventsNotInserted() const;

	const std::map<int32_t, std::map<int32_t, std::map<int32_t, int32_t> > >& channelCounterInserted() const;
	const std::map<int32_t, std::map<int32_t, std::map<int32_t, int32_t> > >& channelCounterNotInserted() const;

	/**
	 * Functions to implement.
	 */
	bool checkEvent(Event const & event);
	void processBadEvent(Event const & event);
	void processEvent(Event const & event);
	void processEvent(EventBlockArrayIterator const & e);
	void processEventNotInserted(Event const & event);

	template<typename Iterator>
	Iterator processBlock(Iterator first, Iterator last, bool lastBlock);
	void finish();

private:
	int32_t _maxNumberOfChannels;

	int32_t _numberOfGoodEvents;
	int32_t _numberOfBadEvents;
	int32_t _numberOfEventsNotInserted;

	std::map<int32_t, std::map<int32_t, std::map<int32_t, int32_t> > > _channelCounterInserted;
	std::map<int32_t, std::map<int32_t, std::map<int32_t, int32_t> > > _channelCounterNotInserted;

	static const int16_t MAX_BITS = 15;
};

template<typename Iterator>
Iterator Insert::processBlock(Iterator first, Iterator last, bool lastBlock) {
	return last;
}

}

#endif
