/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_LIVEREADER_H
#define LSTDPP_LIVEREADER_H

#include "BaseReader.h"

namespace lstdpp128 {

class LiveReader : public BaseReader {

public:
	/**
	 * Constructor.
	 */
	LiveReader(int32_t size);

	/**
	 * Destructor.
	 */
	virtual ~LiveReader();

	/**
	 * Opens the file. Returns false if problem opening the file.
	 */
	virtual bool open(std::string const & fileName);

	/**
	 * Returns the number of elements read. Blocks until the size is read.
	 */
	virtual int32_t read();

	/**
	 * Stops the reader.
	 */
	void stop();

private:
	FileSize getBlockSizeInBytes();

	bool _end;

	static const int32_t WAIT_TIME_US = 10000;
	static const int32_t RETRY = 1000000;
};

}

#endif
