/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Reader.h"

#include <iostream>

using namespace std;

namespace lstdpp128 {

Reader::Reader(int32_t size) {
	init(size);
}

Reader::~Reader() {
	delete [] _boardHeader;
}

int32_t * Reader::boardHeader() const {
	return _boardHeader;
}

bool Reader::open(std::string const & fileName) {

	_dataFile.open(fileName.c_str(), ios::in | ios::binary);

	if (!_dataFile.is_open()) {
		cerr << "problem reading " << fileName << endl;
		return false;
	}

	_dataFile.seekg(0, ios::end);
	_fileSize = _dataFile.tellg();
	_dataFile.seekg(0, ios::beg);

	// Read the 4 fixed elements of the header.
	uint32_t nbEvents = 0;
	uint32_t nbBoards = 0;

	_dataFile.read(reinterpret_cast<char *>(&_version), sizeof(int32_t));
	_dataFile.read(reinterpret_cast<char *>(&_timeBase), sizeof(int32_t));
	_dataFile.read(reinterpret_cast<char *>(&nbEvents), sizeof(int32_t));
	_dataFile.read(reinterpret_cast<char *>(&nbBoards), sizeof(int32_t));

	// Read the board structure.
	_boardHeader = new int32_t[nbBoards];
	_dataFile.read(reinterpret_cast<char *>(_boardHeader), nbBoards * sizeof(uint32_t));

	// Read the list mode context.
	_totalNumberOfBoards = readListModeContextFromBinary(listModeContext, _version, _timeBase, nbEvents, nbBoards, (uint32_t*)_boardHeader);

	return true;
}

int32_t Reader::read() {

	// reading next block
	int32_t blockSize = getBlockSizeInBytes();
	_dataFile.read((char *)_buffer, blockSize);

	return (blockSize / 4);
}

int32_t Reader::getBlockSizeInBytes() {

	FileSize size = _fileSize - _dataFile.tellg();
	if (size < (FileSize)_sizeInBytes) {
		return size;
	}

	return _sizeInBytes;
}

}
