/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Sort.h"

using namespace std;

namespace lstdpp128 {

Sort::Sort() :
	_maxNumberOfChannels(0),
	_outMemblock(0),
	_numberOfGoodEvents(0),
	_numberOfBadEvents(0),
	_numberOfEventsNotInserted(0),
	_maxTime(0),
	_eventId(0) {
}

void Sort::init(int32_t* headerMemoryBlock, uint32_t version, uint32_t timeBase, uint32_t numberOfBoards, int32_t maxBlockSize, std::string const & outFileName) {

	_maxNumberOfChannels = (1 << MAX_BITS);

	_outFileName = outFileName;
	string tempOutFileName = outFileName + ".temp";
	_outDataFile.open(tempOutFileName.c_str());

	cout << "writing " << _outFileName << endl;

	_numberOfGoodEvents = 0;
	_numberOfBadEvents = 0;
	_numberOfEventsNotInserted = 0;

	uint32_t nbEvents = 0;

	// Write the header.
	_outDataFile.write((char *)&version, sizeof(uint32_t));
	_outDataFile.write((char *)&timeBase, sizeof(uint32_t));
	_outDataFile.write((char *)&nbEvents, sizeof(uint32_t));
	_outDataFile.write((char *)&numberOfBoards, sizeof(uint32_t));

	_outDataFile.write((char *)headerMemoryBlock, numberOfBoards * sizeof(uint32_t));

	_maxBlockSize = maxBlockSize;
	_outMemblock = new int32_t[maxBlockSize * 4];

	// comparing number of boards
	BoardIterator b;
	uint32_t boards = 0;
	while (!b.end()) {
		boards++;
		++b;
	}

	cout << "boards " << boards << " " << numberOfBoards << endl;
}

Sort::~Sort() {
	delete [] _outMemblock;
}

int32_t Sort::numberOfGoodEvents() const {
	return _numberOfGoodEvents;
}

int32_t Sort::numberOfBadEvents() const {
	return _numberOfBadEvents;
}

int32_t Sort::numberOfEventsNotInserted() const {
	return _numberOfEventsNotInserted;
}

bool Sort::checkEvent(Event const & event) {

	if (event.crate >= listModeContext.crateBoard.nbCrates) {
		return false;
	}

	if (event.board >= listModeContext.crateBoard.crates[event.crate].nbBoards) {
		return false;
	}

	if (event.channel >= listModeContext.crateBoard.crates[event.crate].boards[event.board].nbChannels) {
		return false;
	}

	return true;
}

void Sort::processBadEvent(Event const & event) {
	_numberOfBadEvents++;
}

void Sort::processEvent(Event const & event) {
	_numberOfGoodEvents++;
}

void Sort::processEvent(EventBlockArrayIterator const & e) {
	_numberOfGoodEvents++;
}

void Sort::processEventNotInserted(Event const & event) {
	_numberOfGoodEvents++;
	_numberOfEventsNotInserted++;
}

void Sort::finish() {

	// Write the number of events.
	// The pointer is 2 uint32 after the beginning of the file.
	_outDataFile.seekp(2 * sizeof(uint32_t), ios::beg);
	_outDataFile.write((char *)&_numberOfGoodEvents, sizeof(uint32_t));

	_outDataFile.close();

	string tempOutFileName = _outFileName + ".temp";
	rename(tempOutFileName.c_str(), _outFileName.c_str());
}

}
