/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_SORT_H
#define LSTDPP_SORT_H

#include <cstring>
#include <algorithm>
#include <fstream>
#include <vector>
#include <time.h>

#include "Event.h"
#include "EventBlockArray.h"

namespace lstdpp128 {

class Sort {

public:
	Sort();
	~Sort();

	void init(int32_t* headerMemoryBlock, uint32_t version, uint32_t timeBase, uint32_t numberOfBoards, int32_t maxBlockSize, std::string const & outFileName);

	int32_t numberOfGoodEvents() const;
	int32_t numberOfBadEvents() const;
	int32_t numberOfEventsNotInserted() const;

	/**
	 * Functions to implement.
	 */
	bool checkEvent(Event const & event);
	void processBadEvent(Event const & event);
	void processEvent(Event const & event);
	void processEvent(EventBlockArrayIterator const & e);
	void processEventNotInserted(Event const & event);

	template<typename Iterator>
	Iterator processBlock(Iterator first, Iterator last, bool lastBlock);
	void finish();

private:
	template<typename Iterator>
	void writeBlock(Iterator first, Iterator last);

	int32_t _maxNumberOfChannels;

	int32_t _maxBlockSize;
	int32_t * _outMemblock;
	std::string _outFileName;
	std::ofstream _outDataFile;

	uint32_t _numberOfGoodEvents;
	int32_t _numberOfBadEvents;
	int32_t _numberOfEventsNotInserted;
	time_type _maxTime;

	int32_t _eventId;

	static const int16_t MAX_BITS = 15;
};

template<typename Iterator>
Iterator Sort::processBlock(Iterator first, Iterator last, bool lastBlock) {
	writeBlock(first, last);

	return last;
}

template<typename Iterator>
void Sort::writeBlock(Iterator first, Iterator last) {

	int32_t * buffer = _outMemblock;
	int32_t index = 0;

	// iterating events
	for (Iterator e = first; e != last; ++e) {

		writeEvent(*e, buffer + index);

		index += 4;
		_eventId++;
	}

	if (index <= _maxBlockSize * 4) {
		_outDataFile.write((char *)_outMemblock, index * sizeof(int32_t));

	} else {
		std::cerr << "cannot write block with size " << index << std::endl;
	}
}

}

#endif
