/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef LSTDPP_RATES_H
#define LSTDPP_RATES_H

#include <cstring>
#include <algorithm>
#include <fstream>
#include <vector>
#include <time.h>
#include <map>

#include "../../Event.h"
#include "../../EventBlockArray.h"

namespace lstdpp128 {

class Rates {

public:
	Rates();
	~Rates();

	void init(int32_t* headerMemoryBlock, uint32_t version, uint32_t timeBase, uint32_t numberOfBoards, int32_t maxBlockSize, std::string const & outFileName);

	int32_t numberOfGoodEvents() const;
	int32_t numberOfBadEvents() const;
	int32_t numberOfEventsNotInserted() const;
	int32_t numberOfEventsInConflict() const;

	/**
	 * Functions to implement.
	 */
	bool checkEvent(Event const & event);
	void processEvent(EventBlockArrayIterator const & e);
	void processBadEvent(Event const & event);
	void processEventNotInserted(Event const & event);

	template<typename Iterator>
	Iterator processBlock(Iterator first, Iterator last, bool lastBlock);
	void finish();
	void printRates();

	int32_t getTotalNumberOfChannels() const;
	double * getChannelRates() const;

private:
	void writeBlock(EventBlock const & block);

	// crate, board - channel offset
	std::map<int32_t, std::map<int32_t, int32_t> > _channelOffset;

	int32_t _totalNbChannels;
	uint64_t * _channelCounters;
	double * _channelRates;
	int32_t _maxBlockSize;

	int32_t * _outMemblock;
	std::string _outFileName;
	std::ofstream _outDataFile;

	int32_t _numberOfGoodEvents;
	int32_t _numberOfBadEvents;
	int32_t _numberOfEventsNotInserted;
	int32_t _numberOfEventsInConflict;

	static const int16_t MAX_BITS = 15;
};

template<typename Iterator>
Iterator Rates::processBlock(Iterator first, Iterator last, bool lastBlock) {

	if (_outFileName != "") {
		int32_t * buffer = _outMemblock;
		int32_t index = 0;

		// iterating events
		for (Iterator e = first; e != last; ++e) {
			writeEvent(*e, buffer + index);
			index += 4;
		}

		if (index <= _maxBlockSize * 4) {
			_outDataFile.write((char *)_outMemblock, index * sizeof(int32_t));

		} else {
			std::cerr << "cannot write block with size " << index << std::endl;
		}
	}

	return last;
}

}

#endif
