
#include "../../algorithms/root/QA.h"

using namespace std;

namespace lstdpp128 {

QA::QA(){
}

QA::~QA(){
	_htimeVsEvt=0; delete _htimeVsEvt;
	_hADCRates=0; delete _hADCRates;
	_hEvents=0; delete _hEvents;
	_hMult=0; delete _hMult;
	_hSyst=0; delete _hSyst;
	_hRelativeTimeTH2=0; delete _hRelativeTimeTH2;
	_hCoincMatrix=0; delete _hCoincMatrix;
}

void QA::Init(const int numberOfADCs, const int numberOfChannels, const int numberOfEvents, const int maxEvent, const int energyMax, const int maxTimeCor, const int matrixSize){

	_numberOfADCs=numberOfADCs;

//MULT
	_hMult = new  TH1I("Multiplicity","Multiplicity distribution; Multiplicity; # of counts",maxEvent,0,maxEvent);

//RATES
	_hADCRates = new  TH1D("ADCRates","ADC Rates; ADC Id.; Rate (s^{-1})", numberOfADCs, 0, numberOfADCs);

//SYSTEM
	_hSyst = new  TH1I("System","System; ;time (s)",2,0,2);
	_hSyst->GetXaxis()->SetBinLabel(1,"Real Time");
	_hSyst->GetXaxis()->SetBinLabel(2,"CPU Time");

//EVENTS

	_hEvents = new TH1D("Events","Events",4,0,4);
	_hEvents->GetXaxis()->SetBinLabel(1,"# of events read");
	_hEvents->GetXaxis()->SetBinLabel(2,"# of events not inserted");
	_hEvents->GetXaxis()->SetBinLabel(3,"# of coincidences truncated");
	_hEvents->GetXaxis()->SetBinLabel(4,"block size (b)");

//TIMEVSEVT
	int newNbBin=numberOfEvents;
	if (numberOfEvents>1000000){
		while(newNbBin>1000000){
			newNbBin=newNbBin/2;
		}
	}
	_htimeVsEvt = new  TH1D("timeVsEvt","Event Id. vs Time ; Event Id.; Time (time stamp unit)", newNbBin, 0., double(numberOfEvents));

//SPECTRA
	for (int iadc=0; iadc<numberOfADCs;iadc++){
		ostringstream ss;
		ss<<iadc;
		string name;
		name="ADCSpectrum"+ss.str();
		TH1I *htemp=new TH1I(const_cast<char*>(name.c_str()),const_cast<char*>(name.c_str()),numberOfChannels,0, numberOfChannels);
		htemp->GetXaxis()->SetTitle("Channel");
		htemp->GetYaxis()->SetTitle("# of counts");
		_hADCSpectra.push_back(htemp);
	}

//SPECTRACAL
	for (int iadc=0; iadc<numberOfADCs;iadc++){
		ostringstream ss;
		ss<<iadc;
		string name;
		name="ADCSpectrumCal"+ss.str();
		TH1I *htemp=new TH1I(const_cast<char*>(name.c_str()),const_cast<char*>(name.c_str()),numberOfChannels,0, energyMax);
		htemp->GetXaxis()->SetTitle("Energy(keV)");
		htemp->GetYaxis()->SetTitle("# of counts");
		_hADCSpectraCal.push_back(htemp);
	}

//SPECTRACALCLEAN
	for (int iadc=0; iadc<numberOfADCs;iadc++){
		ostringstream ss;
		ss<<iadc;
		string name;
		name="ADCSpectrumCalClean"+ss.str();
		TH1I *htemp=new TH1I(const_cast<char*>(name.c_str()),const_cast<char*>(name.c_str()),numberOfChannels,0, energyMax);
		htemp->GetXaxis()->SetTitle("Energy(keV)");
		htemp->GetYaxis()->SetTitle("# of counts");
		_hADCSpectraCalClean.push_back(htemp);
	}

//RELATIVE TIME

	_hRelativeTimeTH2 =  new TH2I("RelativeTimeTH2","Relative Time; ADC Id.; ADC Id.", numberOfADCs, 0, numberOfADCs, numberOfADCs, 0, numberOfADCs);
	for (int iadc1=0; iadc1<numberOfADCs;iadc1++){
		for (int iadc2=0; iadc2<numberOfADCs;iadc2++){
			ostringstream ss1, ss2;
			ss1<<iadc1;
			ss2<<iadc2;
			string name;
			name="hRelativeTimeTH1_"+ss1.str()+"_"+ss2.str();
			TH1I *htemp=new TH1I(const_cast<char*>(name.c_str()),const_cast<char*>(name.c_str()),2*maxTimeCor,-maxTimeCor, maxTimeCor);
			_hRelativeTimeTH1.push_back(htemp);
		}
	}

//COINC MATRIX
	_hCoincMatrix = new TH2I("CoincMatrix","Coincidence Matrix ; Energy (keV); Energy (keV)", matrixSize, 0, energyMax, matrixSize, 0, energyMax);

}

void QA::FillRelativeTime(){
	for (int iadc1=0; iadc1<_numberOfADCs;iadc1++){
		for (int iadc2=0; iadc2<_numberOfADCs;iadc2++){
			Int_t max=_hRelativeTimeTH1[iadc1*_numberOfADCs+iadc2]->GetXaxis()->GetBinCenter(_hRelativeTimeTH1[iadc1*_numberOfADCs+iadc2]->GetMaximumBin());
			_hRelativeTimeTH2->SetBinContent(iadc1+1,iadc2+1,max);
		}
	}
}

}
