/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "Utils.h"

#include <map>

using namespace std;
using namespace boost;

using boost::property_tree::ptree;

namespace lstdpp128 {

void readListModeContextFromPropertyTree(ListModeContext & listModeContext, const boost::property_tree::ptree& data) {

	listModeContext.version = data.get<uint32_t>("version");
	listModeContext.timeBase = data.get<uint32_t>("timeBase");
	listModeContext.nbEvents = data.get<uint32_t>("nbEvents");

	ptree crateBoard = data.get_child("crateboard");

	listModeContext.crateBoard.nbCrates = crateBoard.size();

	// Allocate the crates.
	listModeContext.crateBoard.crates = new Crate[listModeContext.crateBoard.nbCrates];

	// Allocate the boards.
	uint32_t c = 0;
	for (ptree::const_iterator cr = crateBoard.begin(); cr != crateBoard.end(); ++cr) {

		ptree crate = cr->second;

		uint16_t crateNbBoards = crate.size();
		listModeContext.crateBoard.crates[c].nbBoards = crateNbBoards;
		listModeContext.crateBoard.crates[c].boards = new Board[crateNbBoards];

		uint32_t b = 0;
		for (ptree::const_iterator br = crate.begin(); br != crate.end(); ++br) {

			ptree board = br->second;

			listModeContext.crateBoard.crates[c].boards[b].eventType = board.get<uint16_t>("eventType");
			listModeContext.crateBoard.crates[c].boards[b].crate = board.get<uint8_t>("crate");
			listModeContext.crateBoard.crates[c].boards[b].nbChannels = board.get<uint16_t>("nbChannels");
			listModeContext.crateBoard.crates[c].boards[b].boardType = (BoardType)board.get<int>("boardType");

			++b;
		}

		++c;
	}
}

}
