/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "../../CrateBoard.h"
#include "../../EventData.h"
#include <iostream>
#include <iomanip>
#include <cstdlib>

using namespace std;
using namespace lstdpp128;

int main(int argc, char * argv[]) {

	int N = 14;

	BoardType boardTypes[] = {
			CFD,
			C1751_PSD,
			C1724_PHA,
			C1730_PSD,
			C1730_PHA,
			C1740_QDC,
			C1740_QDIV,
			C1725_PHA,
			C1725_PSD,
			GGPICKUP,
			MCC,
			CAMERA,
			GGDMS,
			GGFMCPICKUP
	};

	cout << setw(20) << "name"
			<< setw(10) << "id"
			<< setw(10) << "rel. time"
			<< setw(10) << "time res."
			<< setw(10) << "energy"
			<< setw(10) << "q-short"
			<< setw(10) << "q-long"
			<< setw(10) << "pos. X"
			<< setw(10) << "pos. Y"
			<< setw(10) << "A"
			<< setw(10) << "A+B"
			<< setw(10) << "0-cross."
			<< setw(10) << "flags"
			<< endl;

	for (int i = 0; i < N; ++i) {
		cout << setw(20) << BoardTypeName[boardTypes[i]]
             << setw(10) << (int)boardTypes[i];

		// Relative time.
		if (BoardRelativeTime[boardTypes[i]] == 0) {
			cout << setw(10) << "no";
		}
		else {
			cout << setw(10) << "yes";
		}

        cout << setw(10) << BoardTimeResolution[boardTypes[i]];

		// Energy.
		if (boardEnergyResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardEnergyResolution(boardTypes[i]);
		}

		// Q.
		if (boardQShortResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardQShortResolution(boardTypes[i]);
		}

		if (boardQLongResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardQLongResolution(boardTypes[i]);
		}

		// Position.
		if (boardPositionXResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardPositionXResolution(boardTypes[i]);
		}

		if (boardPositionYResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardPositionYResolution(boardTypes[i]);
		}

		// A B
		if (boardAResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardAResolution(boardTypes[i]);
		}

		if (boardAplusBResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardAplusBResolution(boardTypes[i]);
		}

		// Zero-crossing.
		if (boardZeroCrossingResolution(boardTypes[i]) == 0) {
			cout << setw(10) << "-";
		}
		else {
			cout << setw(10) << boardZeroCrossingResolution(boardTypes[i]);
		}

		// Flags.
		if (boardHasFlags(boardTypes[i]) != 0) {
			cout << setw(10) << "yes";
		}
		else {
			cout << setw(10) << "no";
		}

		cout << endl;
	}

	return EXIT_SUCCESS;
}
