/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cstdlib>
#include <fstream>
#include <sstream>
#include <cstring>
#include <algorithm>
#include <time.h>

#include "../../Event.h"
#include "../../EventArray.h"
#include "../../EventArrayAlgorithm.h"
#include "../../Insert.h"
#include "../../Reader.h"

using namespace std;
using namespace lstdpp128;

int32_t BUFFER_SIZE = 1 << 12;
int32_t BLOCK_SIZE = 1 << 13;

int percentage(int a, int b) {
	return 100.0f * (float)a / (float)b;
}

int main(int argc, char * argv[]) {

	if (argc < 2) {
		cerr << "usage : lstcalibrateinsert128 <file> [minBlockSize]" << endl;
		return EXIT_FAILURE;
	}

	string fileName = argv[1];

	int32_t minBlockSize = BLOCK_SIZE;

	if (argc >= 3) {
		istringstream im(argv[2]);
		im >> minBlockSize;
	}

	cout << "reading " << fileName << ", min block size " << minBlockSize << ", buffer size " << BUFFER_SIZE << endl;

	int32_t blockSize = minBlockSize;

	try {

		while (true) {

			cout << "testing with " << blockSize << endl;

			Reader reader(BUFFER_SIZE);

			bool success = reader.open(fileName);

			if (!success) {
				return EXIT_FAILURE;
			}

			// algorithm
			EventArrayAlgorithm<Insert> algorithm;
			algorithm.processor().init();

			// creating the events
			EventArray events(blockSize);

			int currentPercentage = 0;

			time_t start = time(0);
			time_t t1 = start;

			// reading list mode
			bool ok = true;

			while (true) {
				// reading next block
				int32_t size = reader.read();

				// exiting if no data
				if (size == 0) {
					break;
				}

				// processing block
				algorithm.process(reader.buffer(), size, events);

				// testing if some events were not inserted
				if (algorithm.processor().numberOfEventsNotInserted() > 0) {
					// the size is too small
					ok = false;
					break;
				}
			}

			if (ok) {
				cout << "Block size " << blockSize << " is enough for " << fileName << endl;
				break;
			} else {
				blockSize = (blockSize * 2);
			}
		}

	} catch (bad_alloc &) {
		cout << "Cannot allocate enough memory for processing " << fileName << " without losing events" << endl;
	}

	//cout << "#events not inserted = " << algorithm.processor().numberOfEventsNotInserted() << " (" << percentage(algorithm.processor().numberOfEventsNotInserted(), algorithm.processor().numberOfEvents()) << "%)" << endl;

	return EXIT_SUCCESS;
}
