/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <iostream>
#include <stdint.h>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <cstdio>

using namespace std;

const char * GEN_FILE = "gen.lst";
const int BUFFER_SIZE = 10;
const int NBEVENT=10;
int BOARDS = 2;
int CRATES = 1;
int CHANNELS = 32768;

int Crate[NBEVENT]={0,0,0,0,0,0,0,0,1,1};
int EvtTime[NBEVENT]={0,1,3,7,12,14,15,16,19,25};
int EvtADC[NBEVENT]={0,1,2,3,4,5,6,7,0,1};
int EvtEnergy[NBEVENT]={0,10,20,30,40,50,60,70,80,90};

int main() {

	remove(GEN_FILE);

	ofstream dataFile(GEN_FILE);

	int32_t header[4];
	header[0] = CRATES * BOARDS * 8;
	header[1] = CHANNELS;
	header[2] = 0;
	header[3] = 0; // not set because is evolving

	dataFile.write((char *)header, 4 * sizeof(int32_t));

	int32_t * events = new int32_t[BUFFER_SIZE * 2];

	int32_t * ptr = events;
	//int64_t time = 0;
	long crate = 0;
	long board = 0;
	long adc = 0;
	int32_t energy = 0;

	// loop
	//while (true) {

		for (int32_t i = 0; i < NBEVENT; i++) {

			long rollOver = (EvtTime[i] >> 30);
			*ptr = 0;
			*ptr = (board << 28) | (EvtADC[i] << 25) | (rollOver << 15) | (EvtEnergy[i] & 0x7FFF);

			int32_t value = *ptr;
			long rboard = (value >> 28) & 0xF;
			long radc = (value >> 25) & 0x7;

			ptr++;
			*ptr = (Crate[i] << 30) | (EvtTime[i] & 0x3FFFFFFF);
			ptr++;

			//time++;
			/*
			crate = (3 * i + 1) % CRATES;
			board = (i * i + 1) % BOARDS;
			adc = (3 * i + 1) % 8;
			energy = (5 * i * i * i + 1) % CHANNELS;
			*/
		}

		dataFile.write((char *)events, BUFFER_SIZE * 2 * sizeof(int32_t));

		ptr = events;
		//usleep(period * 1000);
	//}

	return EXIT_SUCCESS;
}
